#include "daq_device_mwpc.h"
#include <mwpcTriggerHandler.h>

#include <unistd.h>
#include <string.h>
#include <netdb.h>

//for the sekect in tge flush routine
#include <sys/select.h>

#define PORT 5000

using namespace std;

int readn(int , char *, int);
int writen(int , char *, int);


daq_device_mwpc::daq_device_mwpc(const int eventtype
				 , const int subeventid
				 , const char *ipaddr
				 , const int trigger)
{

  
  m_eventType  = eventtype;
  m_subeventid = subeventid;
  strcpy ( _ip, ipaddr);
  _s = 0;
  _th = 0;
  _trigger = trigger;

  cout << __LINE__ << "  " << __FILE__ << " event type " << m_eventType << " trigger " << _trigger << endl;

  
  _broken = 0;


  if ( _trigger) 
    {
      cout << __LINE__ << "  " << __FILE__ << " registering triggerhandler " << endl;
      _th  = new mwpcTriggerHandler ();
      registerTriggerHandler ( _th);
    }

}

int  daq_device_mwpc::init()
{

  _broken = 0;

  struct sockaddr_in server_addr;
  struct hostent *p_host;
  p_host = gethostbyname(_ip);

  bzero( (char*) &server_addr, sizeof(server_addr) );
  server_addr.sin_family = AF_INET;
  bcopy(p_host->h_addr, &(server_addr.sin_addr.s_addr), p_host->h_length);
  server_addr.sin_port = htons(PORT);

  if ( ( _s = socket(AF_INET, SOCK_STREAM, 0) ) < 0 )
    {
      _broken =1;
      //cout << __LINE__ << "  " << __FILE__ << " broken 1" << endl;
      perror("mwpc");
      return 1;
    }

  if ( connect(_s, (struct sockaddr*) &server_addr
	       , sizeof(server_addr)) < 0 ) 
    {
      _broken =1;
      // cout << __LINE__ << "  " << __FILE__ << " broken 1" << endl;
      perror("mwpc");
      return 1;
    }

  writeValue(1, 0x20);
  sleep (1);
  writeValue(1, 4);

  if ( _trigger ) _th->setFD( _s);

  _tdcmask = readValue( 0xc);

 
  // and we trigger rearm with our event type so it takes effect
  rearm (m_eventType);

  return 0;
}


// the put_data function

int daq_device_mwpc::put_data(const int etype, int * adr, const int length )
{

  if ( _broken ) 
    {
      cout << __LINE__ << "  " << __FILE__ << " broken ";
      identify();
      return 0; //  we had a catastrophic failure
    }

  int len = 0;

  if (etype != m_eventType )  // not our id
    {
      return 0;
    }

  unsigned int z = 0;

  while ( z != 2) 
  {
    //   cout << __LINE__ << "  " << __FILE__ << " polling SPR, socket = " << _s <<endl;
    z = readValue(0xb) & 0xf;
    usleep(1000);
  }

  sevt =  (subevtdata_ptr) adr;
  // set the initial subevent length
  sevt->sub_length =  SEVTHEADERLENGTH;

  // update id's etc
  sevt->sub_id =  m_subeventid;
  sevt->sub_type=2;
  sevt->sub_decoding = 91;  // 91 is  IDFNALMPWC2"

  sevt->reserved[0] = 0;
  sevt->reserved[1] = 0;

  //  if ( length < max_length(etype) ) 
  //  {
  //    cout << __LINE__ << "  " << __FILE__ << " length " << length <<endl;
  //    return 0;
  //  }

  unsigned int data;

  unsigned int *d = (unsigned int *) &sevt->data;
  *d++ = _tdcmask;

  // rdb
  char rdb[] = { 'r','d','b','\n',0};    // length is 4

  int wordcount = read32bitValue( 0xe, 0xf);
  int bytes_left = wordcount *2;
  
  cout << __LINE__ << "  " << __FILE__ << " bytes to read = " << bytes_left << "  " 
       << hex << wordcount << dec  << endl;

  if (  wordcount/2 + 10 >  length  || wordcount  < 0) 
    {
      cout << __LINE__ << "  " << __FILE__ << " wrong size: " << left << " max is " << length << endl;
      return 0;
    }



  // send the "rdb" command
  writen (_s, rdb , 4);

  // ok, now the data are flowing
  // we read the first 4 bytes and see how many more there are
  len =  readn (_s, (char *) d, bytes_left);

  len = ( len +3) / 4 +1; // the +1 is to account for the tcdmask that I added
 
  sevt->sub_padding = len%2;
  len = len + (len%2);
  sevt->sub_length += len;

  //jy=ust making sure we can still do this - -
  z = readValue(0xb) & 0xf;


  return  sevt->sub_length;
}


void daq_device_mwpc::identify(std::ostream& os) const
{
  
  os  << "MWPC Device  Event Type: " << m_eventType << " Subevent id: " << m_subeventid 
      << " IP: " << _ip;
  if ( _th) os << " trigger ";
  if ( _broken) os << " ** not functional ** ";
  os << endl;

}

int daq_device_mwpc::max_length(const int etype) const
{
  if (etype != m_eventType) return 0;
  return  (256*1024*16 + SEVTHEADERLENGTH+10);
}


int  daq_device_mwpc::endrun()
{
  if ( _s) close (_s);
  _s = 0;
  _tdcmask = 0;
  if ( _trigger ) _th->setFD( 0);
  return 0;
}

// the rearm() function
int  daq_device_mwpc::rearm(const int etype)
{
  return 0;
}


int daq_device_mwpc::read32bitValue(const int adrhigh, const int adrlow)
{

  int h = readValue(adrhigh);
  int l = readValue(adrlow);
  int value = ( h << 16) + l;
  return value;
}

int daq_device_mwpc::readValue(const int adr)
{

  char str[128];
  int l = sprintf ( str, "rdb1 %x 1\n", adr);

  int i;
  int value;
  i = write (_s, str, l);
  l = read ( _s, &value, 2);

  //  cout << hex << " value = " << value << dec << endl;
  return value;
}

int daq_device_mwpc::writeValue(const int adr, const int value)
{

  char str[128];
  int l = sprintf ( str, "wr %x %x\n", adr, value);

  int i;
  i = write (_s, str, l);
  flush_socket();
  return 0;
}

int daq_device_mwpc::flush_socket() const
{

  fd_set flag;
  FD_ZERO(&flag);
  FD_SET(_s, &flag);

  struct timeval t_out;
  t_out.tv_sec = 0;
  t_out.tv_usec = 0;
 
  int dummy;
  int retval = select(_s+1, &flag, 0, 0, &t_out);
  while ( retval >0) 
    {
      cout << "flushing..." << endl;
      read (_s, &dummy, 1);
      retval = select(_s+1, &flag, 0, 0, &t_out);
    }

  return 0;
}


daq_device_mwpc::~daq_device_mwpc()
{
  if ( _th) 
    {
      clearTriggerHandler();
      delete _th;
      _th = 0;
    }
  if (_s) close( _s );
}

int readn (int fd, char *ptr, int nbytes)
{
  int nleft, nread;
  nleft = nbytes;
  while ( nleft>0 )
    {
      nread = read (fd, ptr, nleft);
      if ( nread < 0 ) 
	return nread;
      else if (nread == 0) 
	break;
      nleft -= nread;
      ptr += nread;
    }
  return (nbytes-nleft);
}


int writen (int fd, char *ptr, int nbytes)
{
  int nleft, nwritten;
  nleft = nbytes;
  while ( nleft>0 )
    {
      nwritten = write (fd, ptr, nleft);
      if ( nwritten < 0 ) 
	return nwritten;

      nleft -= nwritten;
      ptr += nwritten;
    }
  return (nbytes-nleft);
}

